import logging
from aiogram import Bot, Dispatcher, Router, types
from aiogram.fsm.context import FSMContext
from aiogram.fsm.storage.memory import MemoryStorage
from aiogram.filters import Command
from aiogram.types import CallbackQuery
from aiogram.utils.keyboard import InlineKeyboardBuilder
from aiogram.fsm.state import State, StatesGroup

from test.api_test import get_order_statuses_with_ids  # Импорт функции для получения заказов
from test.api_test import is_login_exists_in_db

from test.config import token
# Устанавливаем уровень логирования
logging.basicConfig(level=logging.INFO)

# Токен вашего бота
# 
bot = Bot(token=token)
storage = MemoryStorage()
dp = Dispatcher(storage=storage)

# Создаём роутер для обработки команд
router = Router()
dp.include_router(router)

# Состояния для FSM
class AuthStates(StatesGroup):
    waiting_for_login = State()

# Словарь для хранения состояния авторизации и начального сообщения /start
auth_users = {}
start_message_ids = {}

# Команда /start
@router.message(Command(commands=['start']))
async def send_welcome(message: types.Message, state: FSMContext):
    chat_id = message.chat.id

    # Сохраняем message_id команды /start
    start_message_ids[chat_id] = message.message_id

    # Сбрасываем авторизацию
    auth_users[chat_id] = None

    # Создаем инлайн-кнопки "Старт", "Вход" и "Регистрация"
    builder = InlineKeyboardBuilder()
    builder.add(types.InlineKeyboardButton(text="Старт⭐", callback_data="start"))
    builder.add(types.InlineKeyboardButton(text="Вход🔐", callback_data="login"))
    builder.add(types.InlineKeyboardButton(text="Регистрация✍", url="https://moverelab.ru/register/"))

    await message.answer("Привет👋", reply_markup=builder.as_markup())

# Удаление всех сообщений до /start
async def delete_messages_up_to_start(chat_id: int, current_message_id: int):
    start_message_id = start_message_ids.get(chat_id)
    if not start_message_id:
        return

    for message_id in range(start_message_id + 1, current_message_id + 1):
        try:
            await bot.delete_message(chat_id, message_id)
        except Exception as e:
            logging.warning(f"Не удалось удалить сообщение {message_id}: {e}")

# Обработчик нажатия кнопки "Старт"
@router.callback_query(lambda call: call.data == 'start')
async def send_about(call: CallbackQuery):
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)
    builder = InlineKeyboardBuilder()
    builder.add(types.InlineKeyboardButton(text="Назад🔙", callback_data="main_menu"))
    await call.message.answer("Этот бот был создан студентами для лаборатории МИЭМ.", reply_markup=builder.as_markup())

# Обработчик нажатия на "Вход"
@router.callback_query(lambda call: call.data == 'login')
async def ask_for_login(call: CallbackQuery, state: FSMContext):
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)

    builder = InlineKeyboardBuilder()
    builder.add(types.InlineKeyboardButton(text="Назад🔙", callback_data="main_menu"))
    await call.message.answer("Введите ваш логин:", reply_markup=builder.as_markup())
    await state.set_state(AuthStates.waiting_for_login)


# Обработка ввода логина
@router.message(AuthStates.waiting_for_login)
async def get_login(message: types.Message, state: FSMContext):
    await delete_messages_up_to_start(message.chat.id, message.message_id)

    chat_id = message.chat.id
    login = message.text

    # Проверяем, существует ли логин в базе данных
    if is_login_exists_in_db(login):
        # Сохраняем логин в словарь авторизованных пользователей
        auth_users[chat_id] = login

        await message.answer(f"Вы успешно вошли в систему как {login}.")
        await show_profile_menu(chat_id)
        await state.clear()
    else:
        # Если логин не найден
        builder = InlineKeyboardBuilder()
        builder.add(types.InlineKeyboardButton(text="Назад🔙", callback_data="main_menu"))
        builder.add(types.InlineKeyboardButton(text="Попробовать снова", callback_data="login"))  # Кнопка для повторного ввода
        await message.answer(
            "Логин не найден. Пожалуйста, введите корректный логин или вернитесь в главное меню.",
            reply_markup=builder.as_markup()
        )

    await state.clear()


# Отображение меню профиля
async def show_profile_menu(chat_id):
    builder = InlineKeyboardBuilder()
    builder.add(types.InlineKeyboardButton(text="Профиль👤", callback_data="profile"))
    builder.add(types.InlineKeyboardButton(text="Мои заказы✅", callback_data="show_orders"))  # Новая кнопка для заказа
    await bot.send_message(chat_id, "Выберите действие:", reply_markup=builder.as_markup())

# Обработчик нажатия на "Профиль"
@router.callback_query(lambda call: call.data == 'profile')
async def show_profile(call: CallbackQuery):
    # Получаем логин из словаря auth_users
    login = auth_users.get(call.message.chat.id, "Не указан")  # Если логин не найден, показываем "Не указан"
    
    # Удаляем предыдущие сообщения
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)
    
    builder = InlineKeyboardBuilder()
    builder.add(types.InlineKeyboardButton(text="Выйти🔚", callback_data="logout"))
    builder.add(types.InlineKeyboardButton(text="Назад🔙", callback_data="profile_menu"))

    await call.message.answer(f"Вы авторизованы как {login}\n", reply_markup=builder.as_markup())

# Обработчик нажатия на "Назад" из профиля
@router.callback_query(lambda call: call.data == 'profile_menu')
async def back_to_profile_menu(call: CallbackQuery):
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)
    await show_profile_menu(call.message.chat.id)

# Обработчик выхода из профиля
@router.callback_query(lambda call: call.data == 'logout')
async def logout_user(call: CallbackQuery, state: FSMContext):
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)
    chat_id = call.message.chat.id
    auth_users[chat_id] = None  # Забываем пользователя при выходе
    await call.message.answer("Вы вышли из профиля.")
    await send_welcome(call.message, state)

# Обработчик возврата в главное меню
@router.callback_query(lambda call: call.data == 'main_menu')
async def back_to_main_menu(call: CallbackQuery, state: FSMContext):
    await delete_messages_up_to_start(call.message.chat.id, call.message.message_id)
    await send_welcome(call.message, state)

# Обработчик команды /profil
@router.message(Command(commands=['profil']))
async def show_profile_status(message: types.Message):
    chat_id = message.chat.id
    
    # Проверяем, авторизован ли пользователь
    login = auth_users.get(chat_id)
    
    if login:  # Если пользователь авторизован, показываем его логин
        await message.answer(f"Вы авторизованы как {login}.")
    else:  # Если пользователь не авторизован
        await message.answer("Вы не авторизованы. Пожалуйста, войдите в систему.")

# Обработчик нажатия на "Мои заказы"
@router.callback_query(lambda call: call.data == 'show_orders')
async def show_orders(call: CallbackQuery):
    chat_id = call.message.chat.id
    login = auth_users.get(chat_id)

    # Удаляем предыдущие сообщения
    await delete_messages_up_to_start(chat_id, call.message.message_id)
    
    if login:
        user_orders = get_order_statuses_with_ids(login)

        # Перевод статусов
        status_translation = {
            'wait_confirmation': 'Ожидает подтверждения⏳',
            'in_work': 'В работе🛠️',
            'ready': 'Готов✔️',
            'done': 'Получен🚩',
            'canceled': 'Отменен❌'
        }

        # Проверка и вывод заказов
        if user_orders:
            print(user_orders)
            order_details = [
                f"Номер заказа: {order_id}\nСтатус: {status_translation.get(status, 'Неизвестный статус')}"
                for order_id, status in user_orders
            ]
            orders_text = "Ваши заказы:\n" + "\n\n".join(order_details)
        else:
            orders_text = "Заказы отсутствуют."

        

        # Строим меню с кнопкой возврата
        builder = InlineKeyboardBuilder()
        builder.add(types.InlineKeyboardButton(text="Назад🔙", callback_data="profile_menu"))
        
        await call.message.answer(orders_text, reply_markup=builder.as_markup())
    else:
        await call.message.answer("Вы не авторизованы. Пожалуйста, войдите в систему.")

# Запуск бота
if __name__ == '__main__':
    dp.run_polling(bot, skip_updates=True)
